// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Src <string uiname="Source";>;
sampler SrcSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Src);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

texture Src2 <string uiname="Mask";>;
sampler SrcSamp2 = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Src2);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


//source transformation
float4x4 tTex: TEXTUREMATRIX <string uiname="Source Transform";>;

//pin for the Pixelshift
int YPixelShift <string uiname="Y Pixel Shift";> = 300;
int TexHeight <string uiname="Texture Height";> = 600;
float DarkThres <string uiname="Darkness Threshold";> = 0.1f;
float4 CutColor : COLOR <string uiname="Cut Color";> = 0.0f;


//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS( float4 PosO  : POSITION,float4 TexCd : TEXCOORD0 )
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);
    
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 psColorMask(vs2ps In): COLOR
{
    //source texture lookup
    float4 outColor = (0.0,0.0,0.0,1.0);
    float4 col = tex2D(SrcSamp, In.TexCd);
    float4 mask = tex2D(SrcSamp2, In.TexCd);

    
    if (mask.r < CutColor.r+0.01f &&mask.r > CutColor.r-0.01f) {
        if (mask.g < CutColor.g+0.01f &&mask.g > CutColor.g-0.01f) {
            if (mask.b < CutColor.b+0.01f &&mask.b > CutColor.b-0.01f) {
                       outColor = col;
            }
        }
     }
    return outColor;
}



// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique ColorMask
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 psColorMask();
    }
}


technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (SrcSamp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
